
//
//  HSMacro.h
//  HSCore
//
//  Created by Kyle Hankinson on 2018-10-05.
//  Copyright © 2018 Hankinsoft Development, Inc. All rights reserved.
//

#ifndef HSMacro_h
#define HSMacro_h

#import "Defer.h"
#import "MachTimer.h"
#import <objc/runtime.h>

#if !DEBUG
#define HSSAFESLEEP(statement) {error(statement)}
#else
#define HSSAFESLEEP(a)        [NSThread sleepForTimeInterval: (a)]
#endif

#define HSEmptyStringOnNil(a)   (a == nil ? @"" : a)

#define mustOverride() {\
NSString * logString = [NSString stringWithFormat:@"%@:%@ must be overridden in a subclass/category", NSStringFromClass(self.class), NSStringFromSelector(_cmd)];\
NSLog(@"%@", logString);\
@throw [NSException exceptionWithName:NSInvalidArgumentException reason: logString userInfo:nil]; \
}

#define LogEnterExitMeasure() {\
MachTimer * hsMeasureFunctionTimer = [MachTimer startTimer];\
[HSLogHelper.sharedInstance logMessage: @"-- <Enter %@:%@:%d>", NSStringFromClass([self class]), NSStringFromSelector(_cmd), __LINE__];\
pspdf_defer {\
NSLog(@"-- <Exit %@:%@:%d> took %0.2f seconds%@.", NSStringFromClass([self class]), NSStringFromSelector(_cmd), __LINE__, hsMeasureFunctionTimer.elapsedSeconds, hsMeasureFunctionTimer.elapsedSeconds > 1 ? @"(slow)" : @"");\
};\
}

// Compare
#define IsEqual(left,right) ((!left && !right) || (left == right) || [left isEqual: right])
#define IsStringEqual(left,right) ((!left && !right) || [left isEqualToString: right])
#define IsDataEqual(left,right) ((!left && !right) || [left isEqualToData: right])
#define StringOrEmptyOnNil(a) (nil == a ? @"" : a)

#if TARGET_OS_IPHONE
#define DeviceIsIpad()          (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
#else
#define DeviceIsIpad()          (FALSE)
#endif

static inline bool HSValidRangeForString(NSRange range, NSString * string)
{
    if(NSNotFound == range.location)
    {
        return NO;
    }

    if(range.length > string.length)
    {
        return NO;
    } // End of length is larger than the string

    if(NSMaxRange(range) > string.length)
    {
        return NO;
    }
    
    return YES;
} // End of HSValidRangeForString

static inline NSArray<Class> *ClassGetSubclasses(Class parentClass)
{
    int numClasses = objc_getClassList(NULL, 0);
    Class *classes = NULL;
    
    classes = (Class*) malloc(sizeof(Class) * numClasses);
    
    numClasses = objc_getClassList(classes, numClasses);
    
    NSMutableArray *result = [NSMutableArray array];
    for (NSInteger i = 0; i < numClasses; i++) {
        Class superClass = classes[i];
        do{
            superClass = class_getSuperclass(superClass);
        } while(superClass && superClass != parentClass);
        
        if (superClass == nil) {
            continue;
        }
        [result addObject:classes[i]];
    }
    free(classes);
    return result;
}

#endif /* HSMacro_h */
